==> apache-tvm-src-v0.21.0.tar.gz: apache-tvm-src-v0.21.0/src/ir/global_var_supply.cc <==
/*
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *   http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.
 */

/*!
 * \file global_var_supply.cc
 * \brief GlobalVarSupply that can be used to generate unique GlobalVars.
 */
#include "tvm/ir/global_var_supply.h"

#include <tvm/ffi/function.h>

#include <utility>

#include "tvm/ir/expr.h"

namespace tvm {

TVM_FFI_STATIC_INIT_BLOCK({ GlobalVarSupplyNode::RegisterReflection(); });

GlobalVarSupply::GlobalVarSupply(const NameSupply& name_supply,
                                 std::unordered_map<std::string, GlobalVar> name_to_var_map) {
  auto n = make_object<GlobalVarSupplyNode>(name_supply, name_to_var_map);
  data_ = std::move(n);
}

std::string GetModuleName(const IRModule& module) {
  return module->GetAttr<String>(tvm::attr::kModuleName).value_or("tvmgen_default");
}

GlobalVarSupply::GlobalVarSupply(const Array<IRModule>& modules) : GlobalVarSupply() {
  if (!modules.empty()) {
    IRModule first_mod = modules.front();
    this->operator->()->name_supply_->prefix_ = GetModuleName(first_mod);
  }
  for (auto& mod : modules) {
    for (auto kv : mod->functions) {
      this->operator->()->ReserveGlobalVar(kv.first);
    }
  }
}

GlobalVarSupply::GlobalVarSupply(const IRModule module)
    : GlobalVarSupply(Array<IRModule>{module}) {}

void GlobalVarSupplyNode::ReserveGlobalVar(const GlobalVar& var, bool allow_conflict) {
  name_supply_->ReserveName(var->name_hint, false);
  if (!allow_conflict) {
    ICHECK(name_to_var_map_.count(var->name_hint) == 0)
        << "GlobalVar " << var << " conflicts by name in this supply.";
  }
  name_to_var_map_[var->name_hint] = var;
}

GlobalVarSupplyNode::GlobalVarSupplyNode(NameSupply name_supply,
                                         std::unordered_map<std::string, GlobalVar> name_to_var_map)
    : name_supply_(std::move(name_supply)), name_to_var_map_(std::move(name_to_var_map)) {}

GlobalVar GlobalVarSupplyNode::UniqueGlobalFor(const String& name, bool add_prefix) {
  String final_name = name_supply_->ReserveName(name, add_prefix);

  auto it = name_to_var_map_.find(final_name);
  if (it != name_to_var_map_.end()) {
    return it->second;
  } else {
    GlobalVar var = GlobalVar(final_name);
    name_to_var_map_.emplace(final_name, var);
    return var;
  }
}

GlobalVar GlobalVarSupplyNode::FreshGlobal(String name, bool add_prefix) {
  String final_name = name_supply_->FreshName(name, add_prefix);
  ICHECK(name_to_var_map_.find(final_name) == name_to_var_map_.end())
      << "GlobalVar already exists for name " << final_name;
  GlobalVar var = GlobalVar(final_name);
  name_to_var_map_.emplace(final_name, var);
  return var;
}

TVM_REGISTER_NODE_TYPE(GlobalVarSupplyNode);

TVM_FFI_REGISTER_GLOBAL("ir.GlobalVarSupply_NameSupply")
    .set_body_typed([](const NameSupply& name_supply) { return GlobalVarSupply(name_supply); });

TVM_FFI_REGISTER_GLOBAL("ir.GlobalVarSupply_IRModule").set_body_typed([](IRModule mod) {
  return GlobalVarSupply(std::move(mod));
});

TVM_FFI_REGISTER_GLOBAL("ir.GlobalVarSupply_IRModules")
    .set_body_typed([](const Array<IRModule>& mods) { return GlobalVarSupply(mods); });

TVM_FFI_REGISTER_GLOBAL("ir.GlobalVarSupply_FreshGlobal")
    .set_body_method(&GlobalVarSupplyNode::FreshGlobal);

TVM_FFI_REGISTER_GLOBAL("ir.GlobalVarSupply_UniqueGlobalFor")
    .set_body_method(&GlobalVarSupplyNode::UniqueGlobalFor);

TVM_FFI_REGISTER_GLOBAL("ir.GlobalVarSupply_ReserveGlobalVar")
    .set_body_method(&GlobalVarSupplyNode::ReserveGlobalVar);

}  // namespace tvm

==> apache-tvm-src-v0.22.0.tar.gz: apache-tvm-src-v0.22.0/src/ir/global_var_supply.cc <==
/*
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *   http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.
 */

/*!
 * \file global_var_supply.cc
 * \brief GlobalVarSupply that can be used to generate unique GlobalVars.
 */
#include "tvm/ir/global_var_supply.h"

#include <tvm/ffi/function.h>
#include <tvm/ffi/reflection/registry.h>

#include <utility>

#include "tvm/ir/expr.h"

namespace tvm {

TVM_FFI_STATIC_INIT_BLOCK() { GlobalVarSupplyNode::RegisterReflection(); }

GlobalVarSupply::GlobalVarSupply(const NameSupply& name_supply,
                                 std::unordered_map<std::string, GlobalVar> name_to_var_map) {
  auto n = ffi::make_object<GlobalVarSupplyNode>(name_supply, name_to_var_map);
  data_ = std::move(n);
}

std::string GetModuleName(const IRModule& module) {
  return module->GetAttr<ffi::String>(tvm::attr::kModuleName).value_or("tvmgen_default");
}

GlobalVarSupply::GlobalVarSupply(const ffi::Array<IRModule>& modules) : GlobalVarSupply() {
  if (!modules.empty()) {
    IRModule first_mod = modules.front();
    this->operator->()->name_supply_->prefix_ = GetModuleName(first_mod);
  }
  for (auto& mod : modules) {
    for (auto kv : mod->functions) {
      this->operator->()->ReserveGlobalVar(kv.first);
    }
  }
}

GlobalVarSupply::GlobalVarSupply(const IRModule module)
    : GlobalVarSupply(ffi::Array<IRModule>{module}) {}

void GlobalVarSupplyNode::ReserveGlobalVar(const GlobalVar& var, bool allow_conflict) {
  name_supply_->ReserveName(var->name_hint, false);
  if (!allow_conflict) {
    ICHECK(name_to_var_map_.count(var->name_hint) == 0)
        << "GlobalVar " << var << " conflicts by name in this supply.";
  }
  name_to_var_map_[var->name_hint] = var;
}

GlobalVarSupplyNode::GlobalVarSupplyNode(NameSupply name_supply,
                                         std::unordered_map<std::string, GlobalVar> name_to_var_map)
    : name_supply_(std::move(name_supply)), name_to_var_map_(std::move(name_to_var_map)) {}

GlobalVar GlobalVarSupplyNode::UniqueGlobalFor(const ffi::String& name, bool add_prefix) {
  ffi::String final_name = name_supply_->ReserveName(name, add_prefix);

  auto it = name_to_var_map_.find(final_name);
  if (it != name_to_var_map_.end()) {
    return it->second;
  } else {
    GlobalVar var = GlobalVar(final_name);
    name_to_var_map_.emplace(final_name, var);
    return var;
  }
}

GlobalVar GlobalVarSupplyNode::FreshGlobal(ffi::String name, bool add_prefix) {
  ffi::String final_name = name_supply_->FreshName(name, add_prefix);
  ICHECK(name_to_var_map_.find(final_name) == name_to_var_map_.end())
      << "GlobalVar already exists for name " << final_name;
  GlobalVar var = GlobalVar(final_name);
  name_to_var_map_.emplace(final_name, var);
  return var;
}

TVM_FFI_STATIC_INIT_BLOCK() {
  namespace refl = tvm::ffi::reflection;
  refl::GlobalDef()
      .def("ir.GlobalVarSupply_NameSupply",
           [](const NameSupply& name_supply) { return GlobalVarSupply(name_supply); })
      .def("ir.GlobalVarSupply_IRModule",
           [](IRModule mod) { return GlobalVarSupply(std::move(mod)); })
      .def("ir.GlobalVarSupply_IRModules",
           [](const ffi::Array<IRModule>& mods) { return GlobalVarSupply(mods); })
      .def_method("ir.GlobalVarSupply_FreshGlobal", &GlobalVarSupplyNode::FreshGlobal)
      .def_method("ir.GlobalVarSupply_UniqueGlobalFor", &GlobalVarSupplyNode::UniqueGlobalFor)
      .def_method("ir.GlobalVarSupply_ReserveGlobalVar", &GlobalVarSupplyNode::ReserveGlobalVar);
}

}  // namespace tvm
